@@ -2,6 +2,9 @@
 
 #include "../../libnetdata/libnetdata.h"
 
+#include <xenstat.h>
+#include <libxl.h>
+
 #define PLUGIN_XENSTAT_NAME "xenstat.plugin"
 
 #define NETDATA_CHART_PRIO_XENSTAT_NODE_CPUS              30001
@@ -62,15 +65,9 @@ int health_variable_lookup(const char *variable, uint32_t hash, struct rrdcalc *
 char *netdata_configured_host_prefix = "";
 
 // Variables
-
 static int debug = 0;
-
 static int netdata_update_every = 1;
 
-#ifdef HAVE_LIBXENSTAT
-#include <xenstat.h>
-#include <libxl.h>
-
 struct vcpu_metrics {
     unsigned int id;
 
@@ -655,7 +652,7 @@ static void print_domain_network_bytes_chart_definition(char *type, unsigned int
 }
 
 static void print_domain_network_packets_chart_definition(char *type, unsigned int network, int obsolete_flag) {
-    printf("CHART %s.packets_network%u '' 'Network%u Recieved/Sent Packets' 'packets/s' 'network' 'xendomain.packets_network' line %d %d %s %s\n"
+    printf("CHART %s.packets_network%u '' 'Network%u Received/Sent Packets' 'packets/s' 'network' 'xendomain.packets_network' line %d %d %s %s\n"
                        , type
                        , network
                        , network
@@ -683,7 +680,7 @@ static void print_domain_network_errors_chart_definition(char *type, unsigned in
 }
 
 static void print_domain_network_drops_chart_definition(char *type, unsigned int network, int obsolete_flag) {
-    printf("CHART %s.drops_network%u '' 'Network%u Recieve/Transmit Drops' 'drops/s' 'network' 'xendomain.drops_network' line %d %d %s %s\n"
+    printf("CHART %s.drops_network%u '' 'Network%u Receive/Transmit Drops' 'drops/s' 'network' 'xendomain.drops_network' line %d %d %s %s\n"
                        , type
                        , network
                        , network
@@ -861,7 +858,7 @@ static void xenstat_send_domain_metrics() {
                     }
                     printf(
                             "BEGIN %s.bytes_network%u\n"
-                            "SET recieved = %lld\n"
+                            "SET received = %lld\n"
                             "SET sent = %lld\n"
                             "END\n"
                             , type
@@ -878,7 +875,7 @@ static void xenstat_send_domain_metrics() {
                     }
                     printf(
                             "BEGIN %s.packets_network%u\n"
-                            "SET recieved = %lld\n"
+                            "SET received = %lld\n"
                             "SET sent = %lld\n"
                             "END\n"
                             , type
@@ -895,7 +892,7 @@ static void xenstat_send_domain_metrics() {
                     }
                     printf(
                             "BEGIN %s.errors_network%u\n"
-                            "SET recieved = %lld\n"
+                            "SET received = %lld\n"
                             "SET sent = %lld\n"
                             "END\n"
                             , type
@@ -912,7 +909,7 @@ static void xenstat_send_domain_metrics() {
                     }
                     printf(
                             "BEGIN %s.drops_network%u\n"
-                            "SET recieved = %lld\n"
+                            "SET received = %lld\n"
                             "SET sent = %lld\n"
                             "END\n"
                             , type
@@ -1093,14 +1090,3 @@ int main(int argc, char **argv) {
     xenstat_uninit(xhandle);
     info("XENSTAT process exiting");
 }
-
-#else // !HAVE_LIBXENSTAT
-
-int main(int argc, char **argv) {
-    (void)argc;
-    (void)argv;
-
-    fatal("xenstat.plugin is not compiled.");
-}
-
-#endif // !HAVE_LIBXENSTAT
