@@ -31,7 +31,7 @@ RRDHOST *rrdhost_find_by_guid(const char *guid, uint32_t hash) {
     strncpyz(tmp.machine_guid, guid, GUID_LEN);
     tmp.hash_machine_guid = (hash)?hash:simple_hash(tmp.machine_guid);
 
-    return (RRDHOST *)avl_search_lock(&(rrdhost_root_index), (avl *) &tmp);
+    return (RRDHOST *)avl_search_lock(&(rrdhost_root_index), (avl_t *) &tmp);
 }
 
 RRDHOST *rrdhost_find_by_hostname(const char *hostname, uint32_t hash) {
@@ -53,8 +53,8 @@ RRDHOST *rrdhost_find_by_hostname(const char *hostname, uint32_t hash) {
     return NULL;
 }
 
-#define rrdhost_index_add(rrdhost) (RRDHOST *)avl_insert_lock(&(rrdhost_root_index), (avl *)(rrdhost))
-#define rrdhost_index_del(rrdhost) (RRDHOST *)avl_remove_lock(&(rrdhost_root_index), (avl *)(rrdhost))
+#define rrdhost_index_add(rrdhost) (RRDHOST *)avl_insert_lock(&(rrdhost_root_index), (avl_t *)(rrdhost))
+#define rrdhost_index_del(rrdhost) (RRDHOST *)avl_remove_lock(&(rrdhost_root_index), (avl_t *)(rrdhost))
 
 
 // ----------------------------------------------------------------------------
@@ -103,6 +103,28 @@ static inline void rrdhost_init_machine_guid(RRDHOST *host, const char *machine_
     host->hash_machine_guid = simple_hash(host->machine_guid);
 }
 
+void set_host_properties(RRDHOST *host, int update_every, RRD_MEMORY_MODE memory_mode, const char *hostname,
+                         const char *registry_hostname, const char *guid, const char *os, const char *tags,
+                         const char *tzone, const char *program_name, const char *program_version)
+{
+
+    host->rrd_update_every = update_every;
+    host->rrd_memory_mode = memory_mode;
+
+    rrdhost_init_hostname(host, hostname);
+
+    rrdhost_init_machine_guid(host, guid);
+
+    rrdhost_init_os(host, os);
+    rrdhost_init_timezone(host, tzone);
+    rrdhost_init_tags(host, tags);
+
+    host->program_name = strdupz((program_name && *program_name) ? program_name : "unknown");
+    host->program_version = strdupz((program_version && *program_version) ? program_version : "unknown");
+
+    host->registry_hostname = strdupz((registry_hostname && *registry_hostname) ? registry_hostname : host->hostname);
+}
+
 // ----------------------------------------------------------------------------
 // RRDHOST - add a host
 
@@ -127,18 +149,21 @@ RRDHOST *rrdhost_create(const char *hostname,
 ) {
     debug(D_RRDHOST, "Host '%s': adding with guid '%s'", hostname, guid);
 
+#ifdef ENABLE_DBENGINE
+    int is_legacy = (memory_mode == RRD_MEMORY_MODE_DBENGINE) && is_legacy_child(guid);
+#else
+    int is_legacy = 1;
+#endif
     rrd_check_wrlock();
 
+    int is_in_multihost = (memory_mode == RRD_MEMORY_MODE_DBENGINE && !is_legacy);
     RRDHOST *host = callocz(1, sizeof(RRDHOST));
 
-    host->rrd_update_every    = (update_every > 0)?update_every:1;
+    set_host_properties(host, (update_every > 0)?update_every:1, memory_mode, hostname, registry_hostname, guid, os,
+                        tags, timezone, program_name, program_version);
+
     host->rrd_history_entries = align_entries_to_pagesize(memory_mode, entries);
-    host->rrd_memory_mode     = memory_mode;
-#ifdef ENABLE_DBENGINE
-    host->page_cache_mb       = default_rrdeng_page_cache_mb;
-    host->disk_space_mb       = default_rrdeng_disk_quota_mb;
-#endif
-    host->health_enabled      = (memory_mode == RRD_MEMORY_MODE_NONE)? 0 : health_enabled;
+    host->health_enabled      = ((memory_mode == RRD_MEMORY_MODE_NONE)) ? 0 : health_enabled;
 
     host->sender = mallocz(sizeof(*host->sender));
     sender_init(host->sender, host);
@@ -162,18 +187,9 @@ RRDHOST *rrdhost_create(const char *hostname,
 #endif
 
     netdata_rwlock_init(&host->rrdhost_rwlock);
-    netdata_rwlock_init(&host->labels_rwlock);
+    netdata_rwlock_init(&host->labels.labels_rwlock);
 
-    rrdhost_init_hostname(host, hostname);
-    rrdhost_init_machine_guid(host, guid);
-
-    rrdhost_init_os(host, os);
-    rrdhost_init_timezone(host, timezone);
-    rrdhost_init_tags(host, tags);
-
-    host->program_name = strdupz((program_name && *program_name)?program_name:"unknown");
-    host->program_version = strdupz((program_version && *program_version)?program_version:"unknown");
-    host->registry_hostname = strdupz((registry_hostname && *registry_hostname)?registry_hostname:hostname);
+    netdata_mutex_init(&host->aclk_state_lock);
 
     host->system_info = system_info;
 
@@ -222,12 +238,15 @@ RRDHOST *rrdhost_create(const char *hostname,
     }
     else {
         // this is not localhost - append our GUID to localhost path
+        if (is_in_multihost) { // don't append to cache dir in multihost
+            host->cache_dir  = strdupz(netdata_configured_cache_dir);
+        } else {
+            snprintfz(filename, FILENAME_MAX, "%s/%s", netdata_configured_cache_dir, host->machine_guid);
+            host->cache_dir = strdupz(filename);
+        }
 
-        snprintfz(filename, FILENAME_MAX, "%s/%s", netdata_configured_cache_dir, host->machine_guid);
-        host->cache_dir = strdupz(filename);
-
-        if(host->rrd_memory_mode == RRD_MEMORY_MODE_MAP || host->rrd_memory_mode == RRD_MEMORY_MODE_SAVE ||
-           host->rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE) {
+        if((host->rrd_memory_mode == RRD_MEMORY_MODE_MAP || host->rrd_memory_mode == RRD_MEMORY_MODE_SAVE || (
+           host->rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE && is_legacy))) {
             int r = mkdir(host->cache_dir, 0775);
             if(r != 0 && errno != EEXIST)
                 error("Host '%s': cannot create directory '%s'", host->hostname, host->cache_dir);
@@ -271,35 +290,57 @@ RRDHOST *rrdhost_create(const char *hostname,
         health_alarm_log_open(host);
     }
 
+    RRDHOST *t = rrdhost_index_add(host);
+
+    if(t != host) {
+        error("Host '%s': cannot add host with machine guid '%s' to index. It already exists as host '%s' with machine guid '%s'.", host->hostname, host->machine_guid, t->hostname, t->machine_guid);
+        rrdhost_free(host);
+        return NULL;
+    }
+
+    if (likely(!uuid_parse(host->machine_guid, host->host_uuid))) {
+        int rc = sql_store_host(&host->host_uuid, hostname, registry_hostname, update_every, os, timezone, tags);
+        if (unlikely(rc))
+            error_report("Failed to store machine GUID to the database");
+    }
+    else
+        error_report("Host machine GUID %s is not valid", host->machine_guid);
+
     if (host->rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE) {
 #ifdef ENABLE_DBENGINE
-        if (unlikely(-1 == uuid_parse(host->machine_guid, host->host_uuid))) {
-            error("Host machine GUID is not valid.");
-        }
-        host->objects_nr = 1;
-        host->compaction_id = 0;
         char dbenginepath[FILENAME_MAX + 1];
         int ret;
 
         snprintfz(dbenginepath, FILENAME_MAX, "%s/dbengine", host->cache_dir);
         ret = mkdir(dbenginepath, 0775);
-        if(ret != 0 && errno != EEXIST)
+        if (ret != 0 && errno != EEXIST)
             error("Host '%s': cannot create directory '%s'", host->hostname, dbenginepath);
+        else ret = 0; // succeed
+        if (is_legacy) // initialize legacy dbengine instance as needed
+            ret = rrdeng_init(host, &host->rrdeng_ctx, dbenginepath, default_rrdeng_page_cache_mb,
+                              default_rrdeng_disk_quota_mb); // may fail here for legacy dbengine initialization
         else
-            ret = rrdeng_init(host, &host->rrdeng_ctx, dbenginepath, host->page_cache_mb, host->disk_space_mb);
-        if(ret) {
-            error("Host '%s': cannot initialize host with machine guid '%s'. Failed to initialize DB engine at '%s'.",
-                  host->hostname, host->machine_guid, host->cache_dir);
+            host->rrdeng_ctx = &multidb_ctx;
+        if (ret) { // check legacy or multihost initialization success
+            error(
+                "Host '%s': cannot initialize host with machine guid '%s'. Failed to initialize DB engine at '%s'.",
+                host->hostname, host->machine_guid, host->cache_dir);
             rrdhost_free(host);
             host = NULL;
             //rrd_hosts_available++; //TODO: maybe we want this?
 
             return host;
         }
+
 #else
         fatal("RRD_MEMORY_MODE_DBENGINE is not supported in this platform.");
 #endif
     }
+    else {
+#ifdef ENABLE_DBENGINE
+        host->rrdeng_ctx = &multidb_ctx;
+#endif
+    }
 
     // ------------------------------------------------------------------------
     // link it and add it to the index
@@ -316,60 +357,47 @@ RRDHOST *rrdhost_create(const char *hostname,
         else localhost = host;
     }
 
-    RRDHOST *t = rrdhost_index_add(host);
-
-    if(t != host) {
-        error("Host '%s': cannot add host with machine guid '%s' to index. It already exists as host '%s' with machine guid '%s'.", host->hostname, host->machine_guid, t->hostname, t->machine_guid);
-        rrdhost_free(host);
-        host = NULL;
-    }
-    else {
-        info("Host '%s' (at registry as '%s') with guid '%s' initialized"
-                     ", os '%s'"
-                     ", timezone '%s'"
-                     ", tags '%s'"
-                     ", program_name '%s'"
-                     ", program_version '%s'"
-                     ", update every %d"
-                     ", memory mode %s"
-                     ", history entries %ld"
-                     ", streaming %s"
-                     " (to '%s' with api key '%s')"
-                     ", health %s"
-                     ", cache_dir '%s'"
-                     ", varlib_dir '%s'"
-                     ", health_log '%s'"
-                     ", alarms default handler '%s'"
-                     ", alarms default recipient '%s'"
-             , host->hostname
-             , host->registry_hostname
-             , host->machine_guid
-             , host->os
-             , host->timezone
-             , (host->tags)?host->tags:""
-             , host->program_name
-             , host->program_version
-             , host->rrd_update_every
-             , rrd_memory_mode_name(host->rrd_memory_mode)
-             , host->rrd_history_entries
-             , host->rrdpush_send_enabled?"enabled":"disabled"
-             , host->rrdpush_send_destination?host->rrdpush_send_destination:""
-             , host->rrdpush_send_api_key?host->rrdpush_send_api_key:""
-             , host->health_enabled?"enabled":"disabled"
-             , host->cache_dir
-             , host->varlib_dir
-             , host->health_log_filename
-             , host->health_default_exec
-             , host->health_default_recipient
-        );
-    }
+    info("Host '%s' (at registry as '%s') with guid '%s' initialized"
+                 ", os '%s'"
+                 ", timezone '%s'"
+                 ", tags '%s'"
+                 ", program_name '%s'"
+                 ", program_version '%s'"
+                 ", update every %d"
+                 ", memory mode %s"
+                 ", history entries %ld"
+                 ", streaming %s"
+                 " (to '%s' with api key '%s')"
+                 ", health %s"
+                 ", cache_dir '%s'"
+                 ", varlib_dir '%s'"
+                 ", health_log '%s'"
+                 ", alarms default handler '%s'"
+                 ", alarms default recipient '%s'"
+         , host->hostname
+         , host->registry_hostname
+         , host->machine_guid
+         , host->os
+         , host->timezone
+         , (host->tags)?host->tags:""
+         , host->program_name
+         , host->program_version
+         , host->rrd_update_every
+         , rrd_memory_mode_name(host->rrd_memory_mode)
+         , host->rrd_history_entries
+         , host->rrdpush_send_enabled?"enabled":"disabled"
+         , host->rrdpush_send_destination?host->rrdpush_send_destination:""
+         , host->rrdpush_send_api_key?host->rrdpush_send_api_key:""
+         , host->health_enabled?"enabled":"disabled"
+         , host->cache_dir
+         , host->varlib_dir
+         , host->health_log_filename
+         , host->health_default_exec
+         , host->health_default_recipient
+    );
 
     rrd_hosts_available++;
 
-#ifdef ENABLE_DBENGINE
-    if (likely(!is_localhost && host && host->rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE))
-            metalog_commit_update_host(host);
-#endif
     return host;
 }
 
@@ -399,7 +427,7 @@ void rrdhost_update(RRDHOST *host
     UNUSED(rrdpush_api_key);
     UNUSED(rrdpush_send_charts_matching);
 
-    host->health_enabled = health_enabled;
+    host->health_enabled = (mode == RRD_MEMORY_MODE_NONE) ? 0 : health_enabled;
     //host->stream_version = STREAMING_PROTOCOL_CURRENT_VERSION;        Unused?
 
     rrdhost_system_info_free(host->system_info);
@@ -444,10 +472,34 @@ void rrdhost_update(RRDHOST *host
 
     // update host tags
     rrdhost_init_tags(host, tags);
-#ifdef ENABLE_DBENGINE
-    if (likely(host->rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE))
-        metalog_commit_update_host(host);
-#endif
+
+    if (rrdhost_flag_check(host, RRDHOST_FLAG_ARCHIVED)) {
+        rrdhost_flag_clear(host, RRDHOST_FLAG_ARCHIVED);
+        if(host->health_enabled) {
+            int r;
+            char filename[FILENAME_MAX + 1];
+
+            if (host != localhost) {
+                r = mkdir(host->varlib_dir, 0775);
+                if (r != 0 && errno != EEXIST)
+                    error("Host '%s': cannot create directory '%s'", host->hostname, host->varlib_dir);
+            }
+            snprintfz(filename, FILENAME_MAX, "%s/health", host->varlib_dir);
+            r = mkdir(filename, 0775);
+            if(r != 0 && errno != EEXIST)
+                error("Host '%s': cannot create directory '%s'", host->hostname, filename);
+
+            rrdhost_wrlock(host);
+            health_readdir(host, health_user_config_dir(), health_stock_config_dir(), NULL);
+            rrdhost_unlock(host);
+
+            health_alarm_log_load(host);
+            health_alarm_log_open(host);
+        }
+        rrd_hosts_available++;
+        info("Host %s is not in archived mode anymore", host->hostname);
+    }
+
     return;
 }
 
@@ -474,6 +526,13 @@ RRDHOST *rrdhost_find_or_create(
 
     rrd_wrlock();
     RRDHOST *host = rrdhost_find_by_guid(guid, 0);
+    if (unlikely(host && RRD_MEMORY_MODE_DBENGINE != mode && rrdhost_flag_check(host, RRDHOST_FLAG_ARCHIVED))) {
+        /* If a legacy memory mode instantiates all dbengine state must be discarded to avoid inconsistencies */
+        error("Archived host '%s' has memory mode '%s', but the wanted one is '%s'. Discarding archived state.",
+              host->hostname, rrd_memory_mode_name(host->rrd_memory_mode), rrd_memory_mode_name(mode));
+        rrdhost_free(host);
+        host = NULL;
+    }
     if(!host) {
         host = rrdhost_create(
                 hostname
@@ -516,6 +575,12 @@ RRDHOST *rrdhost_find_or_create(
            , rrdpush_send_charts_matching
            , system_info);
     }
+    if (host) {
+        rrdhost_wrlock(host);
+        rrdhost_flag_clear(host, RRDHOST_FLAG_ORPHAN);
+        host->senders_disconnected_time = 0;
+        rrdhost_unlock(host);
+    }
 
     rrdhost_cleanup_orphan_hosts_nolock(host);
 
@@ -523,8 +588,8 @@ RRDHOST *rrdhost_find_or_create(
 
     return host;
 }
-inline int rrdhost_should_be_removed(RRDHOST *host, RRDHOST *protected, time_t now) {
-    if(host != protected
+inline int rrdhost_should_be_removed(RRDHOST *host, RRDHOST *protected_host, time_t now) {
+    if(host != protected_host
        && host != localhost
        && rrdhost_flag_check(host, RRDHOST_FLAG_ORPHAN)
        && host->receiver
@@ -535,17 +600,22 @@ inline int rrdhost_should_be_removed(RRDHOST *host, RRDHOST *protected, time_t n
     return 0;
 }
 
-void rrdhost_cleanup_orphan_hosts_nolock(RRDHOST *protected) {
+void rrdhost_cleanup_orphan_hosts_nolock(RRDHOST *protected_host) {
     time_t now = now_realtime_sec();
 
     RRDHOST *host;
 
 restart_after_removal:
     rrdhost_foreach_write(host) {
-        if(rrdhost_should_be_removed(host, protected, now)) {
+        if(rrdhost_should_be_removed(host, protected_host, now)) {
             info("Host '%s' with machine guid '%s' is obsolete - cleaning up.", host->hostname, host->machine_guid);
 
-            if(rrdhost_flag_check(host, RRDHOST_FLAG_DELETE_ORPHAN_HOST))
+            if (rrdhost_flag_check(host, RRDHOST_FLAG_DELETE_ORPHAN_HOST)
+#ifdef ENABLE_DBENGINE
+                /* don't delete multi-host DB host files */
+                && !(host->rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE && host->rrdeng_ctx == &multidb_ctx)
+#endif
+            )
                 rrdhost_delete_charts(host);
             else
                 rrdhost_save_charts(host);
@@ -565,6 +635,12 @@ int rrd_init(char *hostname, struct rrdhost_system_info *system_info) {
     if (gap_when_lost_iterations_above < 1)
         gap_when_lost_iterations_above = 1;
 
+    if (unlikely(sql_init_database())) {
+        if (default_rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE)
+            return 1;
+        info("Skipping SQLITE metadata initialization since memory mode is not db engine");
+    }
+
     health_init();
 
     rrdpush_init();
@@ -591,8 +667,33 @@ int rrd_init(char *hostname, struct rrdhost_system_info *system_info) {
             , system_info
             , 1
     );
+    if (unlikely(!localhost)) {
+        rrd_unlock();
+        return 1;
+    }
+
+#ifdef ENABLE_DBENGINE
+    char dbenginepath[FILENAME_MAX + 1];
+    int ret;
+    snprintfz(dbenginepath, FILENAME_MAX, "%s/dbengine", localhost->cache_dir);
+    ret = mkdir(dbenginepath, 0775);
+    if (ret != 0 && errno != EEXIST)
+        error("Host '%s': cannot create directory '%s'", localhost->hostname, dbenginepath);
+    else  // Unconditionally create multihost db to support on demand host creation
+        ret = rrdeng_init(NULL, NULL, dbenginepath, default_rrdeng_page_cache_mb, default_multidb_disk_quota_mb);
+    if (ret) {
+        error(
+            "Host '%s' with machine guid '%s' failed to initialize multi-host DB engine instance at '%s'.",
+            localhost->hostname, localhost->machine_guid, localhost->cache_dir);
+        rrdhost_free(localhost);
+        localhost = NULL;
+        rrd_unlock();
+        return 1;
+    }
+#endif
     rrd_unlock();
-	web_client_api_v1_management_init();
+
+    web_client_api_v1_management_init();
     return localhost==NULL;
 }
 
@@ -662,6 +763,7 @@ void rrdhost_system_info_free(struct rrdhost_system_info *system_info) {
         freez(system_info->virt_detection);
         freez(system_info->container);
         freez(system_info->container_detection);
+        freez(system_info->is_k8s_node);
         freez(system_info);
     }
 }
@@ -706,7 +808,10 @@ void rrdhost_free(RRDHOST *host) {
     // release its children resources
 
 #ifdef ENABLE_DBENGINE
-    rrdeng_prepare_exit(host->rrdeng_ctx);
+    if (host->rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE) {
+        if (host->rrdeng_ctx != &multidb_ctx)
+            rrdeng_prepare_exit(host->rrdeng_ctx);
+    }
 #endif
     while(host->rrdset_root)
         rrdset_free(host->rrdset_root);
@@ -738,11 +843,10 @@ void rrdhost_free(RRDHOST *host) {
 
     health_alarm_log_free(host);
 
-    if (host->rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE) {
 #ifdef ENABLE_DBENGINE
+    if (host->rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE && host->rrdeng_ctx != &multidb_ctx)
         rrdeng_exit(host->rrdeng_ctx);
 #endif
-    }
 
     // ------------------------------------------------------------------------
     // remove it from the indexes
@@ -772,8 +876,10 @@ void rrdhost_free(RRDHOST *host) {
     // ------------------------------------------------------------------------
     // free it
 
+    pthread_mutex_destroy(&host->aclk_state_lock);
+    freez(host->aclk_state.claimed_id);
     freez((void *)host->tags);
-    free_host_labels(host->labels);
+    free_label_list(host->labels.head);
     freez((void *)host->os);
     freez((void *)host->timezone);
     freez(host->program_version);
@@ -790,9 +896,10 @@ void rrdhost_free(RRDHOST *host) {
     freez(host->registry_hostname);
     simple_pattern_free(host->rrdpush_send_charts_matching);
     rrdhost_unlock(host);
-    netdata_rwlock_destroy(&host->labels_rwlock);
+    netdata_rwlock_destroy(&host->labels.labels_rwlock);
     netdata_rwlock_destroy(&host->health_log.alarm_log_rwlock);
     netdata_rwlock_destroy(&host->rrdhost_rwlock);
+
     freez(host);
 
     rrd_hosts_available--;
@@ -800,7 +907,9 @@ void rrdhost_free(RRDHOST *host) {
 
 void rrdhost_free_all(void) {
     rrd_wrlock();
-    while(localhost) rrdhost_free(localhost);
+    /* Make sure child-hosts are released before the localhost. */
+    while(localhost->next) rrdhost_free(localhost->next);
+    rrdhost_free(localhost);
     rrd_unlock();
 }
 
@@ -827,55 +936,7 @@ void rrdhost_save_charts(RRDHOST *host) {
     rrdhost_unlock(host);
 }
 
-static int is_valid_label_value(char *value) {
-    while(*value) {
-        if(*value == '"' || *value == '\'' || *value == '*' || *value == '!') {
-            return 0;
-        }
-
-        value++;
-    }
-
-    return 1;
-}
-
-static int is_valid_label_key(char *key) {
-    //Prometheus exporter
-    if(!strcmp(key, "chart") || !strcmp(key, "family")  || !strcmp(key, "dimension"))
-        return 0;
-
-    //Netdata and Prometheus  internal
-    if (*key == '_')
-        return 0;
-
-    while(*key) {
-        if(!(isdigit(*key) || isalpha(*key) || *key == '.' || *key == '_' || *key == '-'))
-            return 0;
-
-        key++;
-    }
-
-    return 1;
-}
-
-char *translate_label_source(LABEL_SOURCE l) {
-    switch (l) {
-        case LABEL_SOURCE_AUTO:
-            return "AUTO";
-        case LABEL_SOURCE_NETDATA_CONF:
-            return "NETDATA.CONF";
-        case LABEL_SOURCE_DOCKER :
-            return "DOCKER";
-        case LABEL_SOURCE_ENVIRONMENT  :
-            return "ENVIRONMENT";
-        case LABEL_SOURCE_KUBERNETES :
-            return "KUBERNETES";
-        default:
-            return "Invalid label source";
-    }
-}
-
-struct label *load_auto_labels()
+static struct label *rrdhost_load_auto_labels(void)
 {
     struct label *label_list = NULL;
 
@@ -927,6 +988,12 @@ struct label *load_auto_labels()
         label_list =
             add_label_to_list(label_list, "_virt_detection", localhost->system_info->virt_detection, LABEL_SOURCE_AUTO);
 
+    if (localhost->system_info->is_k8s_node)
+        label_list =
+            add_label_to_list(label_list, "_is_k8s_node", localhost->system_info->is_k8s_node, LABEL_SOURCE_AUTO);
+
+    label_list = add_aclk_host_labels(label_list);
+
     label_list = add_label_to_list(
         label_list, "_is_parent", (localhost->next || configured_as_parent()) ? "true" : "false", LABEL_SOURCE_AUTO);
 
@@ -937,11 +1004,13 @@ struct label *load_auto_labels()
     return label_list;
 }
 
-static inline int is_valid_label_config_option(char *name, char *value) {
-    return (is_valid_label_key(name) && is_valid_label_value(value) && strcmp(name, "from environment") && strcmp(name, "from kubernetes pods") );
- }
+static inline int rrdhost_is_valid_label_config_option(char *name, char *value)
+{
+    return (is_valid_label_key(name) && is_valid_label_value(value) && strcmp(name, "from environment") &&
+            strcmp(name, "from kubernetes pods"));
+}
 
-struct label *load_config_labels()
+static struct label *rrdhost_load_config_labels()
 {
     int status = config_load(NULL, 1, CONFIG_SECTION_HOST_LABEL);
     if(!status) {
@@ -955,7 +1024,7 @@ struct label *load_config_labels()
         config_section_wrlock(co);
         struct config_option *cv;
         for(cv = co->values; cv ; cv = cv->next) {
-            if( is_valid_label_config_option(cv->name, cv->value)) {
+            if(rrdhost_is_valid_label_config_option(cv->name, cv->value)) {
                 l = add_label_to_list(l, cv->name, cv->value, LABEL_SOURCE_NETDATA_CONF);
                 cv->flags |= CONFIG_VALUE_USED;
             } else {
@@ -969,45 +1038,6 @@ struct label *load_config_labels()
     return l;
 }
 
-typedef enum strip_quotes {
-    DO_NOT_STRIP_QUOTES,
-    STRIP_QUOTES
-} STRIP_QUOTES_OPTION;
-
-typedef enum skip_escaped_characters {
-    DO_NOT_SKIP_ESCAPED_CHARACTERS,
-    SKIP_ESCAPED_CHARACTERS
-} SKIP_ESCAPED_CHARACTERS_OPTION;
-
-static inline void strip_last_symbol(
-    char *str,
-    char symbol,
-    SKIP_ESCAPED_CHARACTERS_OPTION skip_escaped_characters)
-{
-    char *end = str;
-
-    while (*end && *end != symbol) {
-        if (unlikely(skip_escaped_characters && *end == '\\')) {
-            end++;
-            if (unlikely(!*end))
-                break;
-        }
-        end++;
-    }
-    if (likely(*end == symbol))
-        *end = '\0';
-}
-
-static inline char *strip_double_quotes(char *str, SKIP_ESCAPED_CHARACTERS_OPTION skip_escaped_characters)
-{
-    if (*str == '"') {
-        str++;
-        strip_last_symbol(str, '"', skip_escaped_characters);
-    }
-
-    return str;
-}
-
 struct label *parse_simple_tags(
     struct label *label_list,
     const char *tags,
@@ -1097,7 +1127,7 @@ struct label *parse_json_tags(struct label *label_list, const char *tags)
     return label_list;
 }
 
-struct label *load_labels_from_tags()
+static struct label *rrdhost_load_labels_from_tags(void)
 {
     if (!localhost->tags)
         return NULL;
@@ -1141,7 +1171,7 @@ struct label *load_labels_from_tags()
     return label_list;
 }
 
-struct label *load_kubernetes_labels()
+static struct label *rrdhost_load_kubernetes_labels(void)
 {
     struct label *l=NULL;
     char *label_script = mallocz(sizeof(char) * (strlen(netdata_configured_primary_plugins_dir) + strlen("get-kubernetes-labels.sh") + 2));
@@ -1199,104 +1229,26 @@ struct label *load_kubernetes_labels()
     return l;
 }
 
-struct label *create_label(char *key, char *value, LABEL_SOURCE label_source)
+void reload_host_labels(void)
 {
-    size_t key_len = strlen(key), value_len = strlen(value);
-    size_t n = sizeof(struct label) + key_len + 1 + value_len + 1;
-    struct label *result = callocz(1,n);
-    if (result != NULL) {
-        char *c = (char *)result;
-        c += sizeof(struct label);
-        strcpy(c, key);
-        result->key = c;
-        c += key_len + 1;
-        strcpy(c, value);
-        result->value = c;
-        result->label_source = label_source;
-        result->key_hash = simple_hash(result->key);
-    }
-    return result;
-}
-
-void free_host_labels(struct label *labels)
-{
-    while (labels != NULL)
-    {
-        struct label *current = labels;
-        labels = labels->next;
-        freez(current);
-    }
-}
-
-void replace_label_list(RRDHOST *host, struct label *new_labels)
-{
-    rrdhost_check_rdlock(host);
-    netdata_rwlock_wrlock(&host->labels_rwlock);
-    struct label *old_labels = host->labels;
-    host->labels = new_labels;
-    netdata_rwlock_unlock(&host->labels_rwlock);
-
-    free_host_labels(old_labels);
-}
-
-struct label *add_label_to_list(struct label *l, char *key, char *value, LABEL_SOURCE label_source)
-{
-    struct label *lab = create_label(key, value, label_source);
-    lab->next = l;
-    return lab;
-}
-
-int label_list_contains(struct label *head, struct label *check)
-{
-    while (head != NULL)
-    {
-        if (head->key_hash == check->key_hash && !strcmp(head->key, check->key))
-            return 1;
-        head = head->next;
-    }
-    return 0;
-}
-
-/* Create a list with entries from both lists.
-   If any entry in the low priority list is masked by an entry in the high priorty list then delete it.
-*/
-struct label *merge_label_lists(struct label *lo_pri, struct label *hi_pri)
-{
-    struct label *result = hi_pri;
-    while (lo_pri != NULL)
-    {
-        struct label *current = lo_pri;
-        lo_pri = lo_pri->next;
-        if (!label_list_contains(result, current)) {
-            current->next = result;
-            result = current;
-        }
-        else
-            freez(current);
-    }
-    return result;
-}
-
-void reload_host_labels()
-{
-    struct label *from_auto = load_auto_labels();
-    struct label *from_k8s = load_kubernetes_labels();
-    struct label *from_config = load_config_labels();
-    struct label *from_tags = load_labels_from_tags();
+    struct label *from_auto = rrdhost_load_auto_labels();
+    struct label *from_k8s = rrdhost_load_kubernetes_labels();
+    struct label *from_config = rrdhost_load_config_labels();
+    struct label *from_tags = rrdhost_load_labels_from_tags();
 
     struct label *new_labels = merge_label_lists(from_auto, from_k8s);
     new_labels = merge_label_lists(new_labels, from_tags);
     new_labels = merge_label_lists(new_labels, from_config);
 
     rrdhost_rdlock(localhost);
-    replace_label_list(localhost, new_labels);
+    replace_label_list(&localhost->labels, new_labels);
 
     health_label_log_save(localhost);
     rrdhost_unlock(localhost);
 
 /*  TODO-GAPS - fix this so that it looks properly at the state and version of the sender
     if(localhost->rrdpush_send_enabled && localhost->rrdpush_sender_buffer){
-        localhost->labels_flag |= LABEL_FLAG_UPDATE_STREAM;
+        localhost->labels.labels_flag |= LABEL_FLAG_UPDATE_STREAM;
         rrdpush_send_labels(localhost);
     }
 */
@@ -1385,7 +1337,12 @@ void rrdhost_cleanup_all(void) {
 
     RRDHOST *host;
     rrdhost_foreach_read(host) {
-        if(host != localhost && rrdhost_flag_check(host, RRDHOST_FLAG_DELETE_OBSOLETE_CHARTS) && !host->receiver)
+        if (host != localhost && rrdhost_flag_check(host, RRDHOST_FLAG_DELETE_ORPHAN_HOST) && !host->receiver
+#ifdef ENABLE_DBENGINE
+            /* don't delete multi-host DB host files */
+            && !(host->rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE && host->rrdeng_ctx == &multidb_ctx)
+#endif
+        )
             rrdhost_delete_charts(host);
         else
             rrdhost_cleanup_charts(host);
@@ -1414,9 +1371,44 @@ restart_after_removal:
         )) {
 #ifdef ENABLE_DBENGINE
             if(st->rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE) {
+                RRDDIM *rd, *last;
+
                 rrdset_flag_set(st, RRDSET_FLAG_ARCHIVED);
-                while(st->variables)  rrdsetvar_free(st->variables);
-                while(st->alarms)     rrdsetcalc_unlink(st->alarms);
+                while (st->variables)  rrdsetvar_free(st->variables);
+                while (st->alarms)     rrdsetcalc_unlink(st->alarms);
+                rrdset_wrlock(st);
+                for (rd = st->dimensions, last = NULL ; likely(rd) ; ) {
+                    if (rrddim_flag_check(rd, RRDDIM_FLAG_ARCHIVED)) {
+                        last = rd;
+                        rd = rd->next;
+                        continue;
+                    }
+
+                    rrddim_flag_set(rd, RRDDIM_FLAG_ARCHIVED);
+                    while (rd->variables)
+                        rrddimvar_free(rd->variables);
+
+                    if (rrddim_flag_check(rd, RRDDIM_FLAG_OBSOLETE)) {
+                        rrddim_flag_clear(rd, RRDDIM_FLAG_OBSOLETE);
+                        /* only a collector can mark a chart as obsolete, so we must remove the reference */
+                        uint8_t can_delete_metric = rd->state->collect_ops.finalize(rd);
+                        if (can_delete_metric) {
+                            /* This metric has no data and no references */
+                            delete_dimension_uuid(rd->state->metric_uuid);
+                            rrddim_free(st, rd);
+                            if (unlikely(!last)) {
+                                rd = st->dimensions;
+                            }
+                            else {
+                                rd = last->next;
+                            }
+                            continue;
+                        }
+                    }
+                    last = rd;
+                    rd = rd->next;
+                }
+                rrdset_unlock(st);
 
                 debug(D_RRD_CALLS, "RRDSET: Cleaning up remaining chart variables for host '%s', chart '%s'", host->hostname, st->id);
                 rrdvar_free_remaining_variables(host, &st->rrdvar_root_index);
@@ -1426,7 +1418,6 @@ restart_after_removal:
                     /* If the chart still has dimensions don't delete it from the metadata log */
                     continue;
                 }
-                metalog_commit_delete_chart(st);
             }
 #endif
             rrdset_rdlock(st);
@@ -1544,6 +1535,10 @@ int rrdhost_set_system_info_variable(struct rrdhost_system_info *system_info, ch
         freez(system_info->container_detection);
         system_info->container_detection = strdupz(value);
     }
+    else if(!strcmp(name, "NETDATA_HOST_IS_K8S_NODE")){
+        freez(system_info->is_k8s_node);
+        system_info->is_k8s_node = strdupz(value);
+    }
     else if (!strcmp(name, "NETDATA_SYSTEM_CPU_VENDOR"))
         return res;
     else if (!strcmp(name, "NETDATA_SYSTEM_CPU_MODEL"))
