@@ -32,6 +32,7 @@ PACKAGES_UPDATE_IPSETS=${PACKAGES_UPDATE_IPSETS-0}
 PACKAGES_NETDATA_DEMO_SITE=${PACKAGES_NETDATA_DEMO_SITE-0}
 PACKAGES_NETDATA_SENSORS=${PACKAGES_NETDATA_SENSORS-0}
 PACKAGES_NETDATA_DATABASE=${PACKAGES_NETDATA_DATABASE-0}
+PACKAGES_NETDATA_EBPF=${PACKAGES_NETDATA_EBPF-0}
 
 # needed commands
 lsb_release=$(command -v lsb_release 2> /dev/null)
@@ -140,7 +141,7 @@ EOF
 
 release2lsb_release() {
   # loads the given /etc/x-release file
-  # this file is normaly a single line containing something like
+  # this file is normally a single line containing something like
   #
   # X Linux release 1.2.3 (release-name)
   #
@@ -321,8 +322,8 @@ user_picks_distribution() {
   echo >&2 "I found though that the following installers are available:"
   echo >&2
   [ -n "${apt_get}" ] && echo >&2 " - Debian/Ubuntu based (installer is: apt-get)" && opts="apt-get ${opts}"
-  [ -n "${yum}" ] && echo >&2 " - Redhat/Fedora/Centos based (installer is: yum)" && opts="yum ${opts}"
-  [ -n "${dnf}" ] && echo >&2 " - Redhat/Fedora/Centos based (installer is: dnf)" && opts="dnf ${opts}"
+  [ -n "${yum}" ] && echo >&2 " - Red Hat/Fedora/CentOS based (installer is: yum)" && opts="yum ${opts}"
+  [ -n "${dnf}" ] && echo >&2 " - Red Hat/Fedora/CentOS based (installer is: dnf)" && opts="dnf ${opts}"
   [ -n "${zypper}" ] && echo >&2 " - SuSe based (installer is: zypper)" && opts="zypper ${opts}"
   [ -n "${pacman}" ] && echo >&2 " - Arch Linux based (installer is: pacman)" && opts="pacman ${opts}"
   [ -n "${emerge}" ] && echo >&2 " - Gentoo based (installer is: emerge)" && opts="emerge ${opts}"
@@ -641,7 +642,6 @@ declare -A pkg_autogen=(
   # exceptions
   ['centos-6']="WARNING|"
   ['rhel-6']="WARNING|"
-  ['ubuntu-18']="WARNING|"
 )
 
 declare -A pkg_automake=(
@@ -650,6 +650,13 @@ declare -A pkg_automake=(
   ['default']="automake"
 )
 
+# required to bundle libJudy
+declare -A pkg_libtool=(
+  ['gentoo']="sys-devel/libtool"
+  ['clearlinux']="c-basic"
+  ['default']="libtool"
+)
+
 # Required to build libwebsockets and libmosquitto on some systems.
 declare -A pkg_cmake=(
   ['gentoo']="dev-util/cmake"
@@ -893,7 +900,7 @@ declare -A pkg_pkg_config=(
   ['arch']="pkgconfig"
   ['centos']="pkgconfig"
   ['debian']="pkg-config"
-  ['gentoo']="dev-util/pkgconfig"
+  ['gentoo']="virtual/pkgconfig"
   ['sabayon']="virtual/pkgconfig"
   ['rhel']="pkgconfig"
   ['suse']="pkg-config"
@@ -954,6 +961,10 @@ declare -A pkg_python3_mysqldb=(
   ['ubuntu-14.10']="WARNING|"
   ['ubuntu-15.04']="WARNING|"
   ['ubuntu-15.10']="WARNING|"
+  ['centos-7']="python36-mysql"
+  ['centos-8']="python38-mysql"
+  ['rhel-7']="python36-mysql"
+  ['rhel-8']="python38-mysql"
 )
 
 declare -A pkg_python_psycopg2=(
@@ -982,6 +993,11 @@ declare -A pkg_python3_psycopg2=(
   ['clearlinux']="WARNING|"
   ['macos']="WARNING|"
   ['default']="WARNING|"
+
+  ['centos-7']="python3-psycopg2"
+  ['centos-8']="python38-psycopg2"
+  ['rhel-7']="python3-psycopg2"
+  ['rhel-8']="python38-psycopg2"
 )
 
 declare -A pkg_python_pip=(
@@ -996,10 +1012,8 @@ declare -A pkg_python_pip=(
 declare -A pkg_python3_pip=(
   ['alpine']="py3-pip"
   ['arch']="python-pip"
-  ['centos']="WARNING|"
   ['gentoo']="dev-python/pip"
   ['sabayon']="dev-python/pip"
-  ['rhel']="WARNING|"
   ['clearlinux']="python3-basic"
   ['macos']="NOTREQUIRED"
   ['default']="python3-pip"
@@ -1013,6 +1027,7 @@ declare -A pkg_python_pymongo=(
   ['gentoo']="dev-python/pymongo"
   ['suse']="python-pymongo"
   ['clearlinux']="WARNING|"
+  ['rhel']="WARNING|"
   ['macos']="WARNING|"
   ['default']="python-pymongo"
 )
@@ -1025,9 +1040,15 @@ declare -A pkg_python3_pymongo=(
   ['gentoo']="dev-python/pymongo"
   ['suse']="python3-pymongo"
   ['clearlinux']="WARNING|"
+  ['rhel']="WARNING|"
   ['freebsd']="py37-pymongo"
   ['macos']="WARNING|"
   ['default']="python3-pymongo"
+
+  ['centos-7']="python36-pymongo"
+  ['centos-8']="python3-pymongo"
+  ['rhel-7']="python36-pymongo"
+  ['rhel-8']="python3-pymongo"
 )
 
 declare -A pkg_python_requests=(
@@ -1058,6 +1079,11 @@ declare -A pkg_python3_requests=(
   ['clearlinux']="python-extras"
   ['macos']="WARNING|"
   ['default']="WARNING|"
+
+  ['centos-7']="python36-requests"
+  ['centos-8']="python3-requests"
+  ['rhel-7']="python36-requests"
+  ['rhel-8']="python3-requests"
 )
 
 declare -A pkg_lz4=(
@@ -1099,16 +1125,14 @@ declare -A pkg_openssl=(
 )
 
 declare -A pkg_judy=(
-  ['alpine']="WARNING|" # TODO - need to add code to download and install judy for alpine and clearlinux
-  ['clearlinux']="WARNING|"
-  ['macos']="WARNING|"
   ['debian']="libjudy-dev"
   ['ubuntu']="libjudy-dev"
   ['suse']="judy-devel"
   ['gentoo']="dev-libs/judy"
   ['arch']="judy"
   ['freebsd']="Judy"
-  ['default']="Judy-devel"
+  ['fedora']="Judy-devel"
+  ['default']="NOTREQUIRED"
 )
 
 declare -A pkg_python3=(
@@ -1182,6 +1206,28 @@ declare -A pkg_zip=(
   ['default']="zip"
 )
 
+declare -A pkg_libelf=(
+  ['alpine']="elfutils-dev"
+  ['arch']="libelf"
+  ['gentoo']="virtual/libelf"
+  ['sabayon']="virtual/libelf"
+  ['debian']="libelf-dev"
+  ['ubuntu']="libelf-dev"
+  ['fedora']="elfutils-libelf-devel"
+  ['centos']="elfutils-libelf-devel"
+  ['rhel']="elfutils-libelf-devel"
+  ['clearlinux']="devpkg-elfutils"
+  ['suse']="libelf-devel"
+  ['macos']="NOTREQUIRED"
+  ['freebsd']="NOTREQUIRED"
+  ['default']="libelf-devel"
+
+  # exceptions
+  ['alpine-3.5']="libelf-dev"
+  ['alpine-3.4']="libelf-dev"
+  ['alpine-3.3']="libelf-dev"
+)
+
 validate_package_trees() {
   if type -t validate_tree_${tree} > /dev/null; then
     validate_tree_${tree}
@@ -1219,7 +1265,7 @@ suitable_package() {
     return 0
   elif [ -z "${p}" ]; then
     echo >&2 "WARNING"
-    echo >&2 "package ${1} is not availabe in this system."
+    echo >&2 "package ${1} is not available in this system."
     echo >&2
     return 1
   else
@@ -1251,6 +1297,7 @@ packages() {
   suitable_package autoconf-archive
   require_cmd autogen || suitable_package autogen
   require_cmd automake || suitable_package automake
+  require_cmd libtoolize || suitable_package libtool
   require_cmd pkg-config || suitable_package pkg-config
   require_cmd cmake || suitable_package cmake
 
@@ -1317,7 +1364,7 @@ packages() {
   fi
 
   # -------------------------------------------------------------------------
-  # sensors
+  # netdata database
   if [ "${PACKAGES_NETDATA_DATABASE}" -ne 0 ]; then
     suitable_package libuv
     suitable_package lz4
@@ -1325,6 +1372,12 @@ packages() {
     suitable_package judy
   fi
 
+  # -------------------------------------------------------------------------
+  # ebpf plugin
+  if [ "${PACKAGES_NETDATA_EBPF}" -ne 0 ]; then
+    suitable_package libelf
+  fi
+
   # -------------------------------------------------------------------------
   # scripting interpreters for netdata plugins
 
@@ -1352,7 +1405,6 @@ packages() {
   if [ "${PACKAGES_NETDATA_PYTHON3}" -ne 0 ]; then
     require_cmd python3 || suitable_package python3
 
-    suitable_package python3-pymongo
     [ "${PACKAGES_NETDATA_PYTHON_MONGO}" -ne 0 ] && suitable_package python3-pymongo
     # suitable_package python3-requests
     # suitable_package python3-pip
@@ -1440,7 +1492,7 @@ install_apt_get() {
 prompt() {
   if [ "${NON_INTERACTIVE}" -eq 1 ]; then
     echo >&2 "Running in non-interactive mode, assuming yes (y)"
-    echo >&2 " > Would have promptedfor ${1} ..."
+    echo >&2 " > Would have prompted for ${1} ..."
     return 0
   fi
 
@@ -1499,7 +1551,7 @@ validate_tree_centos() {
     echo >&2 " > Checking for PowerTools ..."
     if ! run yum ${sudo} repolist | grep PowerTools; then
       if prompt "PowerTools not found, shall I install it?"; then
-        run ${sudo} yum ${opts} config-manager --set-enabled PowerTools
+        run ${sudo} yum ${opts} config-manager --set-enabled powertools
       fi
     fi
 
@@ -1860,7 +1912,7 @@ EOF
 remote_log() {
   # log success or failure on our system
   # to help us solve installation issues
-  curl > /dev/null 2>&1 -Ss --max-time 3 "https://registry.my-netdata.io/log/installer?status=${1}&error=${2}&distribution=${distribution}&version=${version}&installer=${package_installer}&tree=${tree}&detection=${detection}&netdata=${PACKAGES_NETDATA}&nodejs=${PACKAGES_NETDATA_NODEJS}&python=${PACKAGES_NETDATA_PYTHON}&python3=${PACKAGES_NETDATA_PYTHON3}&mysql=${PACKAGES_NETDATA_PYTHON_MYSQL}&postgres=${PACKAGES_NETDATA_PYTHON_POSTGRES}&pymongo=${PACKAGES_NETDATA_PYTHON_MONGO}&sensors=${PACKAGES_NETDATA_SENSORS}&database=${PACKAGES_NETDATA_DATABASE}&firehol=${PACKAGES_FIREHOL}&fireqos=${PACKAGES_FIREQOS}&iprange=${PACKAGES_IPRANGE}&update_ipsets=${PACKAGES_UPDATE_IPSETS}&demo=${PACKAGES_NETDATA_DEMO_SITE}"
+  curl > /dev/null 2>&1 -Ss --max-time 3 "https://registry.my-netdata.io/log/installer?status=${1}&error=${2}&distribution=${distribution}&version=${version}&installer=${package_installer}&tree=${tree}&detection=${detection}&netdata=${PACKAGES_NETDATA}&nodejs=${PACKAGES_NETDATA_NODEJS}&python=${PACKAGES_NETDATA_PYTHON}&python3=${PACKAGES_NETDATA_PYTHON3}&mysql=${PACKAGES_NETDATA_PYTHON_MYSQL}&postgres=${PACKAGES_NETDATA_PYTHON_POSTGRES}&pymongo=${PACKAGES_NETDATA_PYTHON_MONGO}&sensors=${PACKAGES_NETDATA_SENSORS}&database=${PACKAGES_NETDATA_DATABASE}&ebpf=${PACKAGES_NETDATA_EBPF}&firehol=${PACKAGES_FIREHOL}&fireqos=${PACKAGES_FIREQOS}&iprange=${PACKAGES_IPRANGE}&update_ipsets=${PACKAGES_UPDATE_IPSETS}&demo=${PACKAGES_NETDATA_DEMO_SITE}"
 }
 
 if [ -z "${1}" ]; then
@@ -1935,12 +1987,14 @@ while [ -n "${1}" ]; do
       fi
       PACKAGES_NETDATA_SENSORS=1
       PACKAGES_NETDATA_DATABASE=1
+      PACKAGES_NETDATA_EBPF=1
       ;;
 
     netdata)
       PACKAGES_NETDATA=1
       PACKAGES_NETDATA_PYTHON3=1
       PACKAGES_NETDATA_DATABASE=1
+      PACKAGES_NETDATA_EBPF=1
       ;;
 
     python | netdata-python)
@@ -2023,6 +2077,7 @@ while [ -n "${1}" ]; do
       PACKAGES_UPDATE_IPSETS=1
       PACKAGES_NETDATA_DEMO_SITE=1
       PACKAGES_NETDATA_DATABASE=1
+      PACKAGES_NETDATA_EBPF=1
       ;;
 
     help | -h | --help)
